@@ -5,6 +5,7 @@
 #include "../../../C/CpuArch.h"
 
 #include "../../Common/ComTry.h"
+#include "../../Common/IntToString.h"
 
 #include "../../Windows/PropVariant.h"
 
@@ -278,9 +279,9 @@ Z7_class_CHandler_final: public CHandlerImg
       if (mainName != anotherName && !anotherName.IsEmpty())
       {
         res.Add_Space();
-        res += '(';
+        res.Add_Char('(');
         res += anotherName;
-        res += ')';
+        res.Add_Char(')');
       }
       p = p->Parent;
     }
@@ -689,16 +690,6 @@ static void StringToAString(char *dest, UInt32 val)
   *dest = 0;
 }
 
-static void ConvertByteToHex(unsigned value, char *s)
-{
-  for (int i = 0; i < 2; i++)
-  {
-    unsigned t = value & 0xF;
-    value >>= 4;
-    s[1 - i] = (char)((t < 10) ? ('0' + t) : ('A' + (t - 10)));
-  }
-}
-
 Z7_COM7F_IMF(CHandler::GetArchiveProperty(PROPID propID, PROPVARIANT *value))
 {
   COM_TRY_BEGIN
@@ -754,10 +745,8 @@ Z7_COM7F_IMF(CHandler::GetArchiveProperty(PROPID propID, PROPVARIANT *value))
     }
     case kpidId:
     {
-      char s[32 + 4];
-      for (int i = 0; i < 16; i++)
-        ConvertByteToHex(Footer.Id[i], s + i * 2);
-      s[32] = 0;
+      char s[sizeof(Footer.Id) * 2 + 2];
+      ConvertDataToHex_Upper(s, Footer.Id, sizeof(Footer.Id));
       prop = s;
       break;
     }
@@ -949,13 +938,13 @@ Z7_COM7F_IMF(CHandler::GetStream(UInt32 /* index */, ISequentialInStream **strea
   *stream = NULL;
   if (Footer.IsFixed())
   {
-    CLimitedInStream *streamSpec = new CLimitedInStream;
-    CMyComPtr<ISequentialInStream> streamTemp = streamSpec;
+    CMyComPtr2<ISequentialInStream, CLimitedInStream> streamSpec;
+    streamSpec.Create_if_Empty();
     streamSpec->SetStream(Stream);
     // fixme : check (startOffset = 0)
     streamSpec->InitAndSeek(_startOffset, Footer.CurrentSize);
     RINOK(streamSpec->SeekToStart())
-    *stream = streamTemp.Detach();
+    *stream = streamSpec.Detach();
     return S_OK;
   }
   if (!Footer.ThereIsDynamic() || !AreParentsOK())
