@@ -12,6 +12,20 @@
 namespace NArchive {
 namespace NZip {
 
+HRESULT COutArchive::ClearRestriction()
+{
+  if (SetRestriction)
+    return SetRestriction->SetRestriction(0, 0);
+  return S_OK;
+}
+
+HRESULT COutArchive::SetRestrictionFromCurrent()
+{
+  if (SetRestriction)
+    return SetRestriction->SetRestriction(m_Base + m_CurPos, (UInt64)(Int64)-1);
+  return S_OK;
+}
+
 HRESULT COutArchive::Create(IOutStream *outStream)
 {
   m_CurPos = 0;
@@ -35,42 +49,54 @@ void COutArchive::SeekToCurPos()
 // #define DOES_NEED_ZIP64(v) (v >= 0)
 
 
+Z7_NO_INLINE
 void COutArchive::WriteBytes(const void *data, size_t size)
 {
   m_OutBuffer.WriteBytes(data, size);
   m_CurPos += size;
 }
 
+Z7_NO_INLINE
 void COutArchive::Write8(Byte b)
 {
   m_OutBuffer.WriteByte(b);
   m_CurPos++;
 }
 
+Z7_NO_INLINE
 void COutArchive::Write16(UInt16 val)
 {
   Write8((Byte)val);
   Write8((Byte)(val >> 8));
 }
 
+Z7_NO_INLINE
 void COutArchive::Write32(UInt32 val)
 {
   for (int i = 0; i < 4; i++)
   {
-    Write8((Byte)val);
+    // Write8((Byte)val);
+    m_OutBuffer.WriteByte((Byte)val);
     val >>= 8;
   }
+  m_CurPos += 4;
 }
 
+#define WRITE_CONST_PAIR_16_16(a, b)  { Write32((a) | ((UInt32)(b) << 16)); }
+
+Z7_NO_INLINE
 void COutArchive::Write64(UInt64 val)
 {
   for (int i = 0; i < 8; i++)
   {
-    Write8((Byte)val);
+    // Write8((Byte)val);
+    m_OutBuffer.WriteByte((Byte)val);
     val >>= 8;
   }
+  m_CurPos += 8;
 }
 
+Z7_NO_INLINE
 void COutArchive::WriteExtra(const CExtraBlock &extra)
 {
   FOR_VECTOR (i, extra.SubBlocks)
@@ -97,7 +123,7 @@ void COutArchive::WriteCommonItemInfo(const CLocalItem &item, bool isZip64)
 }
 
 
-#define WRITE_32_VAL_SPEC(__v, __isZip64) Write32((__isZip64) ? 0xFFFFFFFF : (UInt32)(__v));
+#define WRITE_32_VAL_SPEC(_v_, _isZip64_) Write32((_isZip64_) ? 0xFFFFFFFF : (UInt32)(_v_));
 
 
 void COutArchive::WriteUtfName(const CItemOut &item)
@@ -120,11 +146,9 @@ void COutArchive::WriteTimeExtra(const CItemOut &item, bool writeNtfs)
   if (writeNtfs)
   {
     // windows explorer ignores that extra
-    Write16(NFileHeader::NExtraID::kNTFS);
-    Write16(k_Ntfs_ExtraSize);
+    WRITE_CONST_PAIR_16_16(NFileHeader::NExtraID::kNTFS, k_Ntfs_ExtraSize)
     Write32(0); // reserved
-    Write16(NFileHeader::NNtfsExtra::kTagTime);
-    Write16(8 * 3);
+    WRITE_CONST_PAIR_16_16(NFileHeader::NNtfsExtra::kTagTime, 8 * 3)
     WriteNtfsTime(item.Ntfs_MTime);
     WriteNtfsTime(item.Ntfs_ATime);
     WriteNtfsTime(item.Ntfs_CTime);
@@ -134,8 +158,7 @@ void COutArchive::WriteTimeExtra(const CItemOut &item, bool writeNtfs)
   {
     // windows explorer ignores that extra
     // by specification : should we write to local header also?
-    Write16(NFileHeader::NExtraID::kUnixTime);
-    Write16(k_UnixTime_ExtraSize);
+    WRITE_CONST_PAIR_16_16(NFileHeader::NExtraID::kUnixTime, k_UnixTime_ExtraSize)
     const Byte flags = (Byte)((unsigned)1 << NFileHeader::NUnixTime::kMTime);
     Write8(flags);
     UInt32 unixTime;
@@ -192,8 +215,8 @@ void COutArchive::WriteLocalHeader(CItemOut &item, bool needCheck)
     size = 0;
   }
   
-  WRITE_32_VAL_SPEC(packSize, isZip64);
-  WRITE_32_VAL_SPEC(size, isZip64);
+  WRITE_32_VAL_SPEC(packSize, isZip64)
+  WRITE_32_VAL_SPEC(size, isZip64)
 
   Write16((UInt16)item.Name.Len());
 
@@ -203,8 +226,7 @@ void COutArchive::WriteLocalHeader(CItemOut &item, bool needCheck)
 
   if (isZip64)
   {
-    Write16(NFileHeader::NExtraID::kZip64);
-    Write16(8 + 8);
+    WRITE_CONST_PAIR_16_16(NFileHeader::NExtraID::kZip64, 8 + 8)
     Write64(size);
     Write64(packSize);
   }
@@ -249,19 +271,19 @@ void COutArchive::WriteLocalHeader_Replace(CItemOut &item)
 void COutArchive::WriteDescriptor(const CItemOut &item)
 {
   Byte buf[kDataDescriptorSize64];
-  SetUi32(buf, NSignature::kDataDescriptor);
-  SetUi32(buf + 4, item.Crc);
+  SetUi32(buf, NSignature::kDataDescriptor)
+  SetUi32(buf + 4, item.Crc)
   unsigned descriptorSize;
   if (m_IsZip64)
   {
-    SetUi64(buf + 8, item.PackSize);
-    SetUi64(buf + 16, item.Size);
+    SetUi64(buf + 8, item.PackSize)
+    SetUi64(buf + 16, item.Size)
     descriptorSize = kDataDescriptorSize64;
   }
   else
   {
-    SetUi32(buf + 8, (UInt32)item.PackSize);
-    SetUi32(buf + 12, (UInt32)item.Size);
+    SetUi32(buf + 8, (UInt32)item.PackSize)
+    SetUi32(buf + 12, (UInt32)item.Size)
     descriptorSize = kDataDescriptorSize32;
   }
   WriteBytes(buf, descriptorSize);
@@ -283,8 +305,8 @@ void COutArchive::WriteCentralHeader(const CItemOut &item)
   WriteCommonItemInfo(item, isZip64);
   Write32(item.Crc);
 
-  WRITE_32_VAL_SPEC(item.PackSize, isPack64);
-  WRITE_32_VAL_SPEC(item.Size, isUnPack64);
+  WRITE_32_VAL_SPEC(item.PackSize, isPack64)
+  WRITE_32_VAL_SPEC(item.Size, isUnPack64)
 
   Write16((UInt16)item.Name.Len());
   
@@ -306,10 +328,10 @@ void COutArchive::WriteCentralHeader(const CItemOut &item)
   const UInt16 commentSize = (UInt16)item.Comment.Size();
   
   Write16(commentSize);
-  Write16(0); // DiskNumberStart;
+  Write16(0); // DiskNumberStart
   Write16(item.InternalAttrib);
   Write32(item.ExternalAttrib);
-  WRITE_32_VAL_SPEC(item.LocalHeaderPos, isPosition64);
+  WRITE_32_VAL_SPEC(item.LocalHeaderPos, isPosition64)
   WriteBytes((const char *)item.Name, item.Name.Len());
   
   if (isZip64)
@@ -332,8 +354,10 @@ void COutArchive::WriteCentralHeader(const CItemOut &item)
     WriteBytes(item.Comment, commentSize);
 }
 
-void COutArchive::WriteCentralDir(const CObjectVector<CItemOut> &items, const CByteBuffer *comment)
+HRESULT COutArchive::WriteCentralDir(const CObjectVector<CItemOut> &items, const CByteBuffer *comment)
 {
+  RINOK(ClearRestriction())
+  
   const UInt64 cdOffset = GetCurPos();
   FOR_VECTOR (i, items)
     WriteCentralHeader(items[i]);
@@ -341,8 +365,9 @@ void COutArchive::WriteCentralDir(const CObjectVector<CItemOut> &items, const CB
   const UInt64 cdSize = cd64EndOffset - cdOffset;
   const bool cdOffset64 = DOES_NEED_ZIP64(cdOffset);
   const bool cdSize64 = DOES_NEED_ZIP64(cdSize);
-  const bool items64 = items.Size() >= 0xFFFF;
-  const bool isZip64 = (cdOffset64 || cdSize64 || items64);
+  const bool need_Items_64 = items.Size() >= 0xFFFF;
+  const unsigned items16 = (UInt16)(need_Items_64 ? 0xFFFF: items.Size());
+  const bool isZip64 = (cdOffset64 || cdSize64 || need_Items_64);
   
   // isZip64 = true; // to test Zip64
 
@@ -355,10 +380,10 @@ void COutArchive::WriteCentralDir(const CObjectVector<CItemOut> &items, const CB
     // const UInt32 extraSize = 1 << 26;
     // Write64(kEcd64_MainSize + extraSize);
 
-    Write16(45); // made by version
-    Write16(45); // extract version
-    Write32(0); // ThisDiskNumber = 0;
-    Write32(0); // StartCentralDirectoryDiskNumber;;
+    WRITE_CONST_PAIR_16_16(45, // made by version
+        45) // extract version
+    Write32(0); // ThisDiskNumber
+    Write32(0); // StartCentralDirectoryDiskNumber
     Write64((UInt64)items.Size());
     Write64((UInt64)items.Size());
     Write64((UInt64)cdSize);
@@ -373,19 +398,19 @@ void COutArchive::WriteCentralDir(const CObjectVector<CItemOut> &items, const CB
   }
   
   Write32(NSignature::kEcd);
-  Write16(0); // ThisDiskNumber = 0;
-  Write16(0); // StartCentralDirectoryDiskNumber;
-  Write16((UInt16)(items64 ? 0xFFFF: items.Size()));
-  Write16((UInt16)(items64 ? 0xFFFF: items.Size()));
+  WRITE_CONST_PAIR_16_16(0, 0)  // ThisDiskNumber, StartCentralDirectoryDiskNumber
+  Write16((UInt16)items16);
+  Write16((UInt16)items16);
   
-  WRITE_32_VAL_SPEC(cdSize, cdSize64);
-  WRITE_32_VAL_SPEC(cdOffset, cdOffset64);
+  WRITE_32_VAL_SPEC(cdSize, cdSize64)
+  WRITE_32_VAL_SPEC(cdOffset, cdOffset64)
 
   const UInt16 commentSize = (UInt16)(comment ? comment->Size() : 0);
   Write16((UInt16)commentSize);
   if (commentSize != 0)
     WriteBytes((const Byte *)*comment, commentSize);
   m_OutBuffer.FlushWithCheck();
+  return S_OK;
 }
 
 void COutArchive::CreateStreamForCompressing(CMyComPtr<IOutStream> &outStream)
