@@ -48,72 +48,60 @@ static bool ReadDataString(CFSTR fileName, LPCSTR startID,
   NIO::CInFile inFile;
   if (!inFile.Open(fileName))
     return false;
-  const size_t kBufferSize = (1 << 12);
+  const size_t kBufferSize = 1 << 12;
 
   Byte buffer[kBufferSize];
-  const unsigned signatureStartSize = MyStringLen(startID);
-  const unsigned signatureEndSize = MyStringLen(endID);
+  const size_t signatureStartSize = MyStringLen(startID + 1);
+  const size_t signatureEndSize = MyStringLen(endID + 1);
   
   size_t numBytesPrev = 0;
   bool writeMode = false;
-  UInt64 posTotal = 0;
+  UInt32 posTotal = 0;
   for (;;)
   {
-    if (posTotal > (1 << 20))
-      return (stringResult.IsEmpty());
     const size_t numReadBytes = kBufferSize - numBytesPrev;
     size_t processedSize;
     if (!inFile.ReadFull(buffer + numBytesPrev, numReadBytes, processedSize))
       return false;
     if (processedSize == 0)
       return true;
-    const size_t numBytesInBuffer = numBytesPrev + processedSize;
-    UInt32 pos = 0;
+    numBytesPrev += processedSize;
+    size_t pos = 0;
     for (;;)
     {
       if (writeMode)
       {
-        if (pos + signatureEndSize > numBytesInBuffer)
+        if (pos + signatureEndSize > numBytesPrev)
           break;
-        if (memcmp(buffer + pos, endID, signatureEndSize) == 0)
-          return true;
-        const Byte b = buffer[pos];
+        const Byte b = buffer[pos++];
         if (b == 0)
           return false;
+        if (b == ';' && memcmp(buffer + pos, endID + 1, signatureEndSize) == 0)
+          return true;
         stringResult += (char)b;
-        pos++;
       }
       else
       {
-        if (pos + signatureStartSize > numBytesInBuffer)
+        if (pos + signatureStartSize > numBytesPrev)
           break;
-        if (memcmp(buffer + pos, startID, signatureStartSize) == 0)
+        const Byte b = buffer[pos++];
+        if (b == ';' && memcmp(buffer + pos, startID + 1, signatureStartSize) == 0)
         {
           writeMode = true;
           pos += signatureStartSize;
         }
-        else
-          pos++;
       }
     }
-    numBytesPrev = numBytesInBuffer - pos;
-    posTotal += pos;
+    posTotal += (UInt32)pos;
+    if (posTotal > (1 << 21))
+      return stringResult.IsEmpty();
+    numBytesPrev -= pos;
     memmove(buffer, buffer + pos, numBytesPrev);
   }
 }
 
-static char kStartID[] = { ',','!','@','I','n','s','t','a','l','l','@','!','U','T','F','-','8','!', 0 };
-static char kEndID[]   = { ',','!','@','I','n','s','t','a','l','l','E','n','d','@','!', 0 };
-
-static struct CInstallIDInit
-{
-  CInstallIDInit()
-  {
-    kStartID[0] = ';';
-    kEndID[0] = ';';
-  }
-} g_CInstallIDInit;
-
+static const char * const kStartID = ",!@Install@!UTF-8!";
+static const char * const kEndID   = ",!@InstallEnd@!";
 
 #if defined(_WIN32) && defined(_UNICODE) && !defined(_WIN64) && !defined(UNDER_CE)
 #define NT_CHECK_FAIL_ACTION ShowErrorMessage(L"Unsupported Windows version"); return 1;
