@@ -5,6 +5,7 @@
 #include "../../C/CpuArch.h"
 
 #include "../Common/IntToString.h"
+#include "../Common/StringConvert.h"
 
 #ifdef _WIN32
 
@@ -16,9 +17,28 @@
 #include <sys/utsname.h>
 #ifdef __APPLE__
 #include <sys/sysctl.h>
+
 #elif !defined(_AIX)
 
+#if defined(__GLIBC__) && (__GLIBC__ * 100 + __GLIBC_MINOR__ >= 216)
+  #define Z7_GETAUXV_AVAILABLE
+#elif !defined(__QNXNTO__)
+//  #pragma message("=== is not NEW GLIBC === ")
+  #if defined __has_include
+  #if __has_include (<sys/auxv.h>)
+//    #pragma message("=== sys/auxv.h is avail=== ")
+    #define Z7_GETAUXV_AVAILABLE
+  #endif
+  #endif
+#endif
+
+#ifdef Z7_GETAUXV_AVAILABLE
+// #if defined __has_include
+// #if __has_include (<sys/auxv.h>)
 #include <sys/auxv.h>
+#define USE_HWCAP
+// #endif
+// #endif
 
 // #undef AT_HWCAP    // to debug
 // #undef AT_HWCAP2   // to debug
@@ -36,11 +56,39 @@
 #endif
 */
 
+#ifdef USE_HWCAP
+
+#if defined(__FreeBSD__) || defined(__OpenBSD__)
+
+// #if (__FreeBSD__ >= 13) // (FreeBSD 12.01 is required for elf_aux_info() ???)
+static unsigned long MY_getauxval(int aux)
+{
+  unsigned long val;
+  if (elf_aux_info(aux, &val, sizeof(val)))
+    return 0;
+  return val;
+}
+
+#else // ! __FreeBSD__
+
 #ifdef MY_CPU_ARM_OR_ARM64
+  #if defined __has_include
+  #if __has_include (<asm/hwcap.h>)
 #include <asm/hwcap.h>
+  #endif
+  #endif
 #endif
+
+#if defined(AT_HWCAP) || defined(AT_HWCAP2)
+#define MY_getauxval  getauxval
 #endif
 
+#endif // ! __FreeBSD__
+#endif // USE_HWCAP
+#endif // Z7_GETAUXV_AVAILABLE
+
+#endif // !defined(_AIX)
+
 #ifdef __linux__
 #include "../Windows/FileIO.h"
 #endif
@@ -70,13 +118,13 @@ static bool ReadFile_to_Buffer(CFSTR fileName, CByteBuffer &buf)
     return false;
   */
   size_t size = 0;
-  size_t addSize = ((size_t)1 << 12);
+  size_t addSize = (size_t)1 << 12;
   for (;;)
   {
     // printf("\nsize = %d\n", (unsigned)size);
     buf.ChangeSize_KeepData(size + addSize, size);
     size_t processed;
-    if (!file.ReadFull(buf + size, addSize, processed))
+    if (!file.ReadFull(buf.NonConstData() + size, addSize, processed))
       return false;
     if (processed == 0)
     {
@@ -102,38 +150,44 @@ static void PrintHex(AString &s, UInt64 v)
 
 #ifdef MY_CPU_X86_OR_AMD64
 
+Z7_NO_INLINE
 static void PrintCpuChars(AString &s, UInt32 v)
 {
-  for (int j = 0; j < 4; j++)
+  for (unsigned j = 0; j < 4; j++)
   {
-    Byte b = (Byte)(v & 0xFF);
+    const Byte b = (Byte)(v & 0xFF);
     v >>= 8;
     if (b == 0)
       break;
-    s += (char)b;
+    if (b >= 0x20 && b <= 0x7f)
+      s.Add_Char((char)b);
+    else
+    {
+      s.Add_Char('[');
+      char temp[16];
+      ConvertUInt32ToHex(b, temp);
+      s += temp;
+      s.Add_Char(']');
+    }
   }
 }
 
 
-static void x86cpuid_to_String(const Cx86cpuid &c, AString &s, AString &ver)
+static void x86cpuid_to_String(AString &s)
 {
   s.Empty();
 
-  UInt32 maxFunc2 = 0;
-  UInt32 t[3];
+  UInt32 a[4];
+  // cpuid was called already. So we don't check for cpuid availability here
+  z7_x86_cpuid(a, 0x80000000);
 
-  MyCPUID(0x80000000, &maxFunc2, &t[0], &t[1], &t[2]);
-
-  bool fullNameIsAvail = (maxFunc2 >= 0x80000004);
-
-  if (fullNameIsAvail)
+  if (a[0] >= 0x80000004) // if (maxFunc2 >= hi+4) the full name is available
   {
     for (unsigned i = 0; i < 3; i++)
     {
-      UInt32 d[4] = { 0 };
-      MyCPUID(0x80000002 + i, &d[0], &d[1], &d[2], &d[3]);
+      z7_x86_cpuid(a, (UInt32)(0x80000002 + i));
       for (unsigned j = 0; j < 4; j++)
-        PrintCpuChars(s, d[j]);
+        PrintCpuChars(s, a[j]);
     }
   }
 
@@ -141,16 +195,14 @@ static void x86cpuid_to_String(const Cx86cpuid &c, AString &s, AString &ver)
   
   if (s.IsEmpty())
   {
-    for (int i = 0; i < 3; i++)
-      PrintCpuChars(s, c.vendor[i]);
+    z7_x86_cpuid(a, 0);
+    for (unsigned i = 1; i < 4; i++)
+    {
+      const unsigned j = (i ^ (i >> 1));
+      PrintCpuChars(s, a[j]);
+    }
     s.Trim();
   }
-
-  {
-    char temp[32];
-    ConvertUInt32ToHex(c.ver, temp);
-    ver += temp;
-  }
 }
 
 /*
@@ -184,7 +236,7 @@ static void x86cpuid_all_to_String(AString &s)
     {
       char temp[32];
       ConvertUInt32ToHex8Digits(d[i], temp);
-      s += " ";
+      s.Add_Space();
       s += temp;
     }
   }
@@ -215,12 +267,12 @@ static const char * const k_PROCESSOR_ARCHITECTURE[] =
   , "ARM32_ON_WIN64"
 };
 
-#define MY__PROCESSOR_ARCHITECTURE_INTEL 0
-#define MY__PROCESSOR_ARCHITECTURE_AMD64 9
+#define Z7_WIN_PROCESSOR_ARCHITECTURE_INTEL 0
+#define Z7_WIN_PROCESSOR_ARCHITECTURE_AMD64 9
 
 
-#define MY__PROCESSOR_INTEL_PENTIUM  586
-#define MY__PROCESSOR_AMD_X8664      8664
+#define Z7_WIN_PROCESSOR_INTEL_PENTIUM  586
+#define Z7_WIN_PROCESSOR_AMD_X8664      8664
 
 /*
 static const CUInt32PCharPair k_PROCESSOR[] =
@@ -303,19 +355,20 @@ static const char * const k_PF[] =
 #endif
 
 
-#ifdef _WIN32
-
-static void PrintPage(AString &s, UInt32 v)
+static void PrintPage(AString &s, UInt64 v)
 {
-  if ((v & 0x3FF) == 0)
+  const char *t = "B";
+  if ((v & 0x3ff) == 0)
   {
-    s.Add_UInt32(v >> 10);
-    s += "K";
+    v >>= 10;
+    t = "KB";
   }
-  else
-    s.Add_UInt32(v >> 10);
+  s.Add_UInt64(v);
+  s += t;
 }
 
+#ifdef _WIN32
+
 static AString TypeToString2(const char * const table[], unsigned num, UInt32 value)
 {
   char sz[16];
@@ -330,7 +383,7 @@ static AString TypeToString2(const char * const table[], unsigned num, UInt32 va
   return (AString)p;
 }
 
-// #if defined(_7ZIP_LARGE_PAGES) || defined(_WIN32)
+// #if defined(Z7_LARGE_PAGES) || defined(_WIN32)
 // #ifdef _WIN32
 void PrintSize_KMGT_Or_Hex(AString &s, UInt64 v)
 {
@@ -342,32 +395,32 @@ void PrintSize_KMGT_Or_Hex(AString &s, UInt64 v)
   }}}}
   else
   {
+    // s += "0x";
     PrintHex(s, v);
     return;
   }
-  char temp[32];
-  ConvertUInt64ToString(v, temp);
-  s += temp;
+  s.Add_UInt64(v);
   if (c)
-    s += c;
+    s.Add_Char(c);
+  s.Add_Char('B');
 }
 // #endif
 // #endif
 
 static void SysInfo_To_String(AString &s, const SYSTEM_INFO &si)
 {
-  s += TypeToString2(k_PROCESSOR_ARCHITECTURE, ARRAY_SIZE(k_PROCESSOR_ARCHITECTURE), si.wProcessorArchitecture);
+  s += TypeToString2(k_PROCESSOR_ARCHITECTURE, Z7_ARRAY_SIZE(k_PROCESSOR_ARCHITECTURE), si.wProcessorArchitecture);
 
-  if (!( (si.wProcessorArchitecture == MY__PROCESSOR_ARCHITECTURE_INTEL && si.dwProcessorType == MY__PROCESSOR_INTEL_PENTIUM)
-      || (si.wProcessorArchitecture == MY__PROCESSOR_ARCHITECTURE_AMD64 && si.dwProcessorType == MY__PROCESSOR_AMD_X8664)))
+  if (!( (si.wProcessorArchitecture == Z7_WIN_PROCESSOR_ARCHITECTURE_INTEL && si.dwProcessorType == Z7_WIN_PROCESSOR_INTEL_PENTIUM)
+      || (si.wProcessorArchitecture == Z7_WIN_PROCESSOR_ARCHITECTURE_AMD64 && si.dwProcessorType == Z7_WIN_PROCESSOR_AMD_X8664)))
   {
-    s += " ";
-    // s += TypePairToString(k_PROCESSOR, ARRAY_SIZE(k_PROCESSOR), si.dwProcessorType);
+    s.Add_Space();
+    // s += TypePairToString(k_PROCESSOR, Z7_ARRAY_SIZE(k_PROCESSOR), si.dwProcessorType);
     s.Add_UInt32(si.dwProcessorType);
   }
-  s += " ";
+  s.Add_Space();
   PrintHex(s, si.wProcessorLevel);
-  s += ".";
+  s.Add_Dot();
   PrintHex(s, si.wProcessorRevision);
   if ((UInt64)si.dwActiveProcessorMask + 1 != ((UInt64)1 << si.dwNumberOfProcessors))
   if ((UInt64)si.dwActiveProcessorMask + 1 != 0 || si.dwNumberOfProcessors != sizeof(UInt64) * 8)
@@ -375,7 +428,7 @@ static void SysInfo_To_String(AString &s, const SYSTEM_INFO &si)
     s += " act:";
     PrintHex(s, si.dwActiveProcessorMask);
   }
-  s += " cpus:";
+  s += " threads:";
   s.Add_UInt32(si.dwNumberOfProcessors);
   if (si.dwPageSize != 1 << 12)
   {
@@ -387,15 +440,15 @@ static void SysInfo_To_String(AString &s, const SYSTEM_INFO &si)
     s += " gran:";
     PrintPage(s, si.dwAllocationGranularity);
   }
-  s += " ";
+  s.Add_Space();
 
-  DWORD_PTR minAdd = (DWORD_PTR)si.lpMinimumApplicationAddress;
+  const DWORD_PTR minAdd = (DWORD_PTR)si.lpMinimumApplicationAddress;
   UInt64 maxSize = (UInt64)(DWORD_PTR)si.lpMaximumApplicationAddress + 1;
   const UInt32 kReserveSize = ((UInt32)1 << 16);
   if (minAdd != kReserveSize)
   {
     PrintSize_KMGT_Or_Hex(s, minAdd);
-    s += "-";
+    s.Add_Minus();
   }
   else
   {
@@ -419,7 +472,7 @@ static void Add_sysctlbyname_to_String(const char *name, AString &s)
 {
   size_t bufSize = 256;
   char buf[256];
-  if (My_sysctlbyname_Get(name, &buf, &bufSize) == 0)
+  if (z7_sysctlbyname_Get(name, &buf, &bufSize) == 0)
     s += buf;
 }
 #endif
@@ -440,12 +493,14 @@ void GetSysInfo(AString &s1, AString &s2)
     }
     
     #if !defined(_WIN64) && !defined(UNDER_CE)
-    Func_GetNativeSystemInfo fn_GetNativeSystemInfo = (Func_GetNativeSystemInfo)(void *)GetProcAddress(
-        GetModuleHandleA("kernel32.dll"), "GetNativeSystemInfo");
-    if (fn_GetNativeSystemInfo)
+    const
+    Func_GetNativeSystemInfo fn = Z7_GET_PROC_ADDRESS(
+    Func_GetNativeSystemInfo, GetModuleHandleA("kernel32.dll"),
+        "GetNativeSystemInfo");
+    if (fn)
     {
       SYSTEM_INFO si2;
-      fn_GetNativeSystemInfo(&si2);
+      fn(&si2);
       // if (memcmp(&si, &si2, sizeof(si)) != 0)
       {
         // s += " - ";
@@ -457,17 +512,14 @@ void GetSysInfo(AString &s1, AString &s2)
 }
 
 
-void GetCpuName(AString &s);
-
 static void AddBracedString(AString &dest, AString &src)
 {
   if (!src.IsEmpty())
   {
-    AString s;
-    s += '(';
-    s += src;
-    s += ')';
-    dest.Add_OptSpaced(s);
+    dest.Add_Space_if_NotEmpty();
+    dest.Add_Char('(');
+    dest += src;
+    dest.Add_Char(')');
   }
 }
 
@@ -478,6 +530,28 @@ struct CCpuName
   AString Microcode;
   AString LargePages;
 
+#ifdef _WIN32
+  UInt32 MHz;
+
+#ifdef MY_CPU_ARM64
+#define Z7_SYS_INFO_SHOW_ARM64_REGS
+#endif
+#ifdef Z7_SYS_INFO_SHOW_ARM64_REGS
+  bool Arm64_ISAR0_EL1_Defined;
+  UInt64 Arm64_ISAR0_EL1;
+#endif
+#endif
+
+#ifdef _WIN32
+  CCpuName():
+      MHz(0)
+#ifdef Z7_SYS_INFO_SHOW_ARM64_REGS
+    , Arm64_ISAR0_EL1_Defined(false)
+    , Arm64_ISAR0_EL1(0)
+#endif
+    {}
+#endif
+
   void Fill();
 
   void Get_Revision_Microcode_LargePages(AString &s)
@@ -485,48 +559,154 @@ struct CCpuName
     s.Empty();
     AddBracedString(s, Revision);
     AddBracedString(s, Microcode);
-    s.Add_OptSpaced(LargePages);
+#ifdef _WIN32
+    if (MHz != 0)
+    {
+      s.Add_Space_if_NotEmpty();
+      s.Add_UInt32(MHz);
+      s += " MHz";
+    }
+#endif
+    if (!LargePages.IsEmpty())
+      s.Add_OptSpaced(LargePages);
   }
+
+#ifdef Z7_SYS_INFO_SHOW_ARM64_REGS
+  void Get_Registers(AString &s)
+  {
+    if (Arm64_ISAR0_EL1_Defined)
+    {
+      // ID_AA64ISAR0_EL1
+      s.Add_OptSpaced("cp4030:");
+      PrintHex(s, Arm64_ISAR0_EL1);
+      {
+        const unsigned sha2 = ((unsigned)(Arm64_ISAR0_EL1 >> 12) & 0xf) - 1;
+        if (sha2 < 2)
+        {
+          s += ":SHA256";
+          if (sha2)
+            s += ":SHA512";
+        }
+      }
+    }
+  }
+#endif
 };
 
 void CCpuName::Fill()
 {
-  CpuName.Empty();
-  Revision.Empty();
-  Microcode.Empty();
-  LargePages.Empty();
+  // CpuName.Empty();
+  // Revision.Empty();
+  // Microcode.Empty();
+  // LargePages.Empty();
 
   AString &s = CpuName;
 
   #ifdef MY_CPU_X86_OR_AMD64
   {
-    Cx86cpuid cpuid;
-    if (x86cpuid_CheckAndRead(&cpuid))
-    {
-      x86cpuid_to_String(cpuid, s, Revision);
-    }
-    else
-    {
-    #ifdef MY_CPU_AMD64
-    s += "x64";
-    #else
-    s += "x86";
+    #if !defined(MY_CPU_AMD64)
+    if (z7_x86_cpuid_GetMaxFunc())
     #endif
+    {
+      x86cpuid_to_String(s);
+      {
+        UInt32 a[4];
+        z7_x86_cpuid(a, 1);
+        char temp[16];
+        ConvertUInt32ToHex(a[0], temp);
+        Revision += temp;
+      }
     }
   }
   #elif defined(__APPLE__)
   {
     Add_sysctlbyname_to_String("machdep.cpu.brand_string", s);
   }
+  #elif defined(MY_CPU_E2K) && defined(Z7_MCST_LCC_VERSION) && (Z7_MCST_LCC_VERSION >= 12323)
+  {
+    s += "mcst ";
+    s += __builtin_cpu_name();
+    s.Add_Space();
+    s += __builtin_cpu_arch();
+  }
   #endif
 
 
+#ifdef _WIN32
+  {
+    NRegistry::CKey key;
+    if (key.Open(HKEY_LOCAL_MACHINE, TEXT("HARDWARE\\DESCRIPTION\\System\\CentralProcessor\\0"), KEY_READ) == ERROR_SUCCESS)
+    {
+      // s.Empty(); // for debug
+      {
+        CSysString name;
+        if (s.IsEmpty())
+        if (key.QueryValue(TEXT("ProcessorNameString"), name) == ERROR_SUCCESS)
+        {
+          s += GetAnsiString(name);
+        }
+        if (key.QueryValue(TEXT("Identifier"), name) == ERROR_SUCCESS)
+        {
+          if (!Revision.IsEmpty())
+            Revision += " : ";
+          Revision += GetAnsiString(name);
+        }
+      }
+#ifdef _WIN32
+      key.GetValue_UInt32_IfOk(TEXT("~MHz"), MHz);
+#ifdef Z7_SYS_INFO_SHOW_ARM64_REGS
+/*
+mapping arm64 registers to Windows registry:
+CP 4000: MIDR_EL1
+CP 4020: ID_AA64PFR0_EL1
+CP 4021: ID_AA64PFR1_EL1
+CP 4028: ID_AA64DFR0_EL1
+CP 4029: ID_AA64DFR1_EL1
+CP 402C: ID_AA64AFR0_EL1
+CP 402D: ID_AA64AFR1_EL1
+CP 4030: ID_AA64ISAR0_EL1
+CP 4031: ID_AA64ISAR1_EL1
+CP 4038: ID_AA64MMFR0_EL1
+CP 4039: ID_AA64MMFR1_EL1
+CP 403A: ID_AA64MMFR2_EL1
+*/
+      if (key.GetValue_UInt64_IfOk(TEXT("CP 4030"), Arm64_ISAR0_EL1) == ERROR_SUCCESS)
+        Arm64_ISAR0_EL1_Defined = true;
+#endif
+#endif
+      LONG res[2];
+      CByteBuffer bufs[2];
+      res[0] = key.QueryValue_Binary(TEXT("Previous Update Revision"), bufs[0]);
+      res[1] = key.QueryValue_Binary(TEXT("Update Revision"),          bufs[1]);
+      if (res[0] == ERROR_SUCCESS || res[1] == ERROR_SUCCESS)
+      {
+        for (unsigned i = 0; i < 2; i++)
+        {
+          if (i == 1)
+            Microcode += "->";
+          if (res[i] != ERROR_SUCCESS)
+            continue;
+          const CByteBuffer &buf = bufs[i];
+          if (buf.Size() == 8)
+          {
+            const UInt32 high = GetUi32(buf);
+            if (high != 0)
+            {
+              PrintHex(Microcode, high);
+              Microcode.Add_Dot();
+            }
+            PrintHex(Microcode, GetUi32(buf + 4));
+          }
+        }
+      }
+    }
+  }
+#endif
+
   if (s.IsEmpty())
   {
-    #ifdef MY_CPU_LE
-      s += "LE";
-    #elif defined(MY_CPU_BE)
-      s += "BE";
+    #ifdef MY_CPU_NAME
+      s += MY_CPU_NAME;
     #endif
   }
   
@@ -534,15 +714,15 @@ void CCpuName::Fill()
   {
     AString s2;
     UInt32 v = 0;
-    if (My_sysctlbyname_Get_UInt32("machdep.cpu.core_count", &v) == 0)
+    if (z7_sysctlbyname_Get_UInt32("machdep.cpu.core_count", &v) == 0)
     {
       s2.Add_UInt32(v);
-      s2 += 'C';
+      s2.Add_Char('C');
     }
-    if (My_sysctlbyname_Get_UInt32("machdep.cpu.thread_count", &v) == 0)
+    if (z7_sysctlbyname_Get_UInt32("machdep.cpu.thread_count", &v) == 0)
     {
       s2.Add_UInt32(v);
-      s2 += 'T';
+      s2.Add_Char('T');
     }
     if (!s2.IsEmpty())
     {
@@ -552,53 +732,7 @@ void CCpuName::Fill()
   }
   #endif
 
-  
-  #ifdef _WIN32
-  {
-    NRegistry::CKey key;
-    if (key.Open(HKEY_LOCAL_MACHINE, TEXT("HARDWARE\\DESCRIPTION\\System\\CentralProcessor\\0"), KEY_READ) == ERROR_SUCCESS)
-    {
-      LONG res[2];
-      CByteBuffer bufs[2];
-      {
-        for (int i = 0; i < 2; i++)
-        {
-          UInt32 size = 0;
-          res[i] = key.QueryValue(i == 0 ?
-            TEXT("Previous Update Revision") :
-            TEXT("Update Revision"), bufs[i], size);
-          if (res[i] == ERROR_SUCCESS)
-            if (size != bufs[i].Size())
-              res[i] = ERROR_SUCCESS + 1;
-        }
-      }
-      if (res[0] == ERROR_SUCCESS || res[1] == ERROR_SUCCESS)
-      {
-        for (int i = 0; i < 2; i++)
-        {
-          if (i == 1)
-            Microcode += "->";
-          if (res[i] != ERROR_SUCCESS)
-            continue;
-          const CByteBuffer &buf = bufs[i];
-          if (buf.Size() == 8)
-          {
-            UInt32 high = GetUi32(buf);
-            if (high != 0)
-            {
-              PrintHex(Microcode, high);
-              Microcode += ".";
-            }
-            PrintHex(Microcode, GetUi32(buf + 4));
-          }
-        }
-      }
-    }
-  }
-  #endif
-
-
-  #ifdef _7ZIP_LARGE_PAGES
+  #ifdef Z7_LARGE_PAGES
   Add_LargePages_String(LargePages);
   #endif
 }
@@ -608,16 +742,16 @@ void AddCpuFeatures(AString &s)
 {
   #ifdef _WIN32
   // const unsigned kNumFeatures_Extra = 32; // we check also for unknown features
-  // const unsigned kNumFeatures = ARRAY_SIZE(k_PF) + kNumFeatures_Extra;
+  // const unsigned kNumFeatures = Z7_ARRAY_SIZE(k_PF) + kNumFeatures_Extra;
   const unsigned kNumFeatures = 64;
   UInt64 flags = 0;
   for (unsigned i = 0; i < kNumFeatures; i++)
   {
-    if (IsProcessorFeaturePresent(i))
+    if (IsProcessorFeaturePresent((DWORD)i))
     {
       flags += (UInt64)1 << i;
       // s.Add_Space_if_NotEmpty();
-      // s += TypeToString2(k_PF, ARRAY_SIZE(k_PF), i);
+      // s += TypeToString2(k_PF, Z7_ARRAY_SIZE(k_PF), i);
     }
   }
   s.Add_OptSpaced("f:");
@@ -626,11 +760,10 @@ void AddCpuFeatures(AString &s)
   #elif defined(__APPLE__)
   {
     UInt32 v = 0;
-    if (My_sysctlbyname_Get_UInt32("hw.pagesize", &v) == 0)
+    if (z7_sysctlbyname_Get_UInt32("hw.pagesize", &v) == 0)
     {
-      s += "PageSize:";
-      s.Add_UInt32(v >> 10);
-      s += "KB";
+      s.Add_OptSpaced("PageSize:");
+      PrintPage(s, v);
     }
   }
 
@@ -639,10 +772,8 @@ void AddCpuFeatures(AString &s)
   const long v = sysconf(_SC_PAGESIZE);
   if (v != -1)
   {
-    s.Add_Space_if_NotEmpty();
-    s += "PageSize:";
-    s.Add_UInt32((UInt32)(v >> 10));
-    s += "KB";
+    s.Add_OptSpaced("PageSize:");
+    PrintPage(s, (unsigned long)v);
   }
 
   #if !defined(_AIX)
@@ -659,11 +790,11 @@ void AddCpuFeatures(AString &s)
     const int pos = s2.Find('[');
     if (pos >= 0)
     {
-      const int pos2 = s2.Find(']', pos + 1);
+      const int pos2 = s2.Find(']', (unsigned)pos + 1);
       if (pos2 >= 0)
       {
-        s2.DeleteFrom(pos2);
-        s2.DeleteFrontal(pos + 1);
+        s2.DeleteFrom((unsigned)pos2);
+        s2.DeleteFrontal((unsigned)pos + 1);
       }
     }
     s += s2;
@@ -676,12 +807,21 @@ void AddCpuFeatures(AString &s)
   #ifdef AT_HWCAP
   s.Add_OptSpaced("hwcap:");
   {
-    unsigned long h = getauxval(AT_HWCAP);
+    unsigned long h = MY_getauxval(AT_HWCAP);
     PrintHex(s, h);
     #ifdef MY_CPU_ARM64
+#ifndef HWCAP_SHA3
+#define HWCAP_SHA3    (1 << 17)
+#endif
+#ifndef HWCAP_SHA512
+#define HWCAP_SHA512  (1 << 21)
+// #pragma message("=== HWCAP_SHA512 define === ")
+#endif
     if (h & HWCAP_CRC32)  s += ":CRC32";
     if (h & HWCAP_SHA1)   s += ":SHA1";
     if (h & HWCAP_SHA2)   s += ":SHA2";
+    if (h & HWCAP_SHA3)   s += ":SHA3";
+    if (h & HWCAP_SHA512) s += ":SHA512";
     if (h & HWCAP_AES)    s += ":AES";
     if (h & HWCAP_ASIMD)  s += ":ASIMD";
     #elif defined(MY_CPU_ARM)
@@ -692,7 +832,7 @@ void AddCpuFeatures(AString &s)
  
   #ifdef AT_HWCAP2
   {
-    unsigned long h = getauxval(AT_HWCAP2);
+    unsigned long h = MY_getauxval(AT_HWCAP2);
     #ifndef MY_CPU_ARM
     if (h != 0)
     #endif
@@ -716,16 +856,21 @@ void AddCpuFeatures(AString &s)
 #ifdef _WIN32
 #ifndef UNDER_CE
 
+Z7_DIAGNOSTIC_IGNORE_CAST_FUNCTION
+
 EXTERN_C_BEGIN
 typedef void (WINAPI * Func_RtlGetVersion) (OSVERSIONINFOEXW *);
 EXTERN_C_END
 
 static BOOL My_RtlGetVersion(OSVERSIONINFOEXW *vi)
 {
-  HMODULE ntdll = ::GetModuleHandleW(L"ntdll.dll");
+  const HMODULE ntdll = ::GetModuleHandleW(L"ntdll.dll");
   if (!ntdll)
     return FALSE;
-  Func_RtlGetVersion func = (Func_RtlGetVersion)(void *)GetProcAddress(ntdll, "RtlGetVersion");
+  const
+  Func_RtlGetVersion func = Z7_GET_PROC_ADDRESS(
+  Func_RtlGetVersion, ntdll,
+      "RtlGetVersion");
   if (!func)
     return FALSE;
   func(vi);
@@ -752,18 +897,18 @@ void GetOsInfoText(AString &sRes)
         s += "Windows";
         if (vi.dwPlatformId != VER_PLATFORM_WIN32_NT)
           s.Add_UInt32(vi.dwPlatformId);
-        s += " "; s.Add_UInt32(vi.dwMajorVersion);
-        s += "."; s.Add_UInt32(vi.dwMinorVersion);
-        s += " "; s.Add_UInt32(vi.dwBuildNumber);
+        s.Add_Space(); s.Add_UInt32(vi.dwMajorVersion);
+        s.Add_Dot();   s.Add_UInt32(vi.dwMinorVersion);
+        s.Add_Space(); s.Add_UInt32(vi.dwBuildNumber);
 
         if (vi.wServicePackMajor != 0 || vi.wServicePackMinor != 0)
         {
           s += " SP:"; s.Add_UInt32(vi.wServicePackMajor);
-          s += "."; s.Add_UInt32(vi.wServicePackMinor);
+          s.Add_Dot(); s.Add_UInt32(vi.wServicePackMinor);
         }
         // s += " Suite:"; PrintHex(s, vi.wSuiteMask);
         // s += " Type:"; s.Add_UInt32(vi.wProductType);
-        // s += " "; s += GetOemString(vi.szCSDVersion);
+        // s.Add_Space(); s += GetOemString(vi.szCSDVersion);
       }
       /*
       {
@@ -793,6 +938,17 @@ void GetOsInfoText(AString &sRes)
     #endif  // _WIN32
 
     sRes += s;
+  #ifdef MY_CPU_X86_OR_AMD64
+  {
+    AString s2;
+    GetVirtCpuid(s2);
+    if (!s2.IsEmpty())
+    {
+      sRes += " : ";
+      sRes += s2;
+    }
+  }
+  #endif
 }
 
 
@@ -824,13 +980,18 @@ void GetSystemInfoText(AString &sRes)
       }
     }
     {
-      AString s;
-      GetCpuName(s);
+      AString s, registers;
+      GetCpuName_MultiLine(s, registers);
       if (!s.IsEmpty())
       {
         sRes += s;
         sRes.Add_LF();
       }
+      if (!registers.IsEmpty())
+      {
+        sRes += registers;
+        sRes.Add_LF();
+      }
     }
     /*
     #ifdef MY_CPU_X86_OR_AMD64
@@ -848,20 +1009,8 @@ void GetSystemInfoText(AString &sRes)
 }
 
 
-void GetCpuName(AString &s);
-void GetCpuName(AString &s)
-{
-  CCpuName cpuName;
-  cpuName.Fill();
-  s = cpuName.CpuName;
-  AString s2;
-  cpuName.Get_Revision_Microcode_LargePages(s2);
-  s.Add_OptSpaced(s2);
-}
-
-
-void GetCpuName_MultiLine(AString &s);
-void GetCpuName_MultiLine(AString &s)
+void GetCpuName_MultiLine(AString &s, AString &registers);
+void GetCpuName_MultiLine(AString &s, AString &registers)
 {
   CCpuName cpuName;
   cpuName.Fill();
@@ -873,45 +1022,230 @@ void GetCpuName_MultiLine(AString &s)
     s.Add_LF();
     s += s2;
   }
+  registers.Empty();
+#ifdef Z7_SYS_INFO_SHOW_ARM64_REGS
+  cpuName.Get_Registers(registers);
+#endif
 }
 
+
+#ifdef MY_CPU_X86_OR_AMD64
+
+void GetVirtCpuid(AString &s)
+{
+  const UInt32 kHv = 0x40000000;
+
+  Z7_IF_X86_CPUID_SUPPORTED
+  {
+    UInt32 a[4];
+    z7_x86_cpuid(a, kHv);
+    
+    if (a[0] < kHv || a[0] >= kHv + (1 << 16))
+      return;
+    {
+      {
+        for (unsigned j = 1; j < 4; j++)
+          PrintCpuChars(s, a[j]);
+      }
+    }
+    if (a[0] >= kHv + 1)
+    {
+      UInt32 d[4];
+      z7_x86_cpuid(d, kHv + 1);
+      s += " : ";
+      PrintCpuChars(s, d[0]);
+      if (a[0] >= kHv + 2)
+      {
+        z7_x86_cpuid(d, kHv + 2);
+        s += " : ";
+        s.Add_UInt32(d[1] >> 16);
+        s.Add_Dot();  s.Add_UInt32(d[1] & 0xffff);
+        s.Add_Dot();  s.Add_UInt32(d[0]);
+        s.Add_Dot();  s.Add_UInt32(d[2]);
+        s.Add_Dot();  s.Add_UInt32(d[3] >> 24);
+        s.Add_Dot();  s.Add_UInt32(d[3] & 0xffffff);
+      }
+      /*
+      if (a[0] >= kHv + 5)
+      {
+        z7_x86_cpuid(d, kHv + 5);
+        s += " : ";
+        s.Add_UInt32(d[0]);
+        s += "p";
+        s.Add_UInt32(d[1]);
+        s += "t";
+      }
+      */
+    }
+  }
+}
+
+#endif
+
+
 void GetCompiler(AString &s)
 {
+  #ifdef __clang__
+    s += " CLANG ";
+    s.Add_UInt32(__clang_major__);
+    s.Add_Dot();
+    s.Add_UInt32(__clang_minor__);
+    s.Add_Dot();
+    s.Add_UInt32(__clang_patchlevel__);
+  #endif
+
+  #ifdef __xlC__
+    s += " XLC ";
+    s.Add_UInt32(__xlC__ >> 8);
+    s.Add_Dot();
+    s.Add_UInt32(__xlC__ & 0xFF);
+    #ifdef __xlC_ver__
+      s.Add_Dot();
+      s.Add_UInt32(__xlC_ver__ >> 8);
+      s.Add_Dot();
+      s.Add_UInt32(__xlC_ver__ & 0xFF);
+    #endif
+  #endif
+
+  // #define __LCC__ 126
+  // #define __LCC_MINOR__ 20
+  // #define __MCST__ 1
+  #ifdef __MCST__
+    s += " MCST";
+  #endif
+  #ifdef __LCC__
+    s += " LCC ";
+    s.Add_UInt32(__LCC__ / 100);
+    s.Add_Dot();
+    s.Add_UInt32(__LCC__ % 100 / 10);
+    s.Add_UInt32(__LCC__ % 10);
+    #ifdef __LCC_MINOR__
+      s.Add_Dot();
+      s.Add_UInt32(__LCC_MINOR__ / 10);
+      s.Add_UInt32(__LCC_MINOR__ % 10);
+    #endif
+  #endif
+
+  // #define __EDG_VERSION__ 602
+  #ifdef __EDG_VERSION__
+    s += " EDG ";
+    s.Add_UInt32(__EDG_VERSION__ / 100);
+    s.Add_Dot();
+    s.Add_UInt32(__EDG_VERSION__ % 100 / 10);
+    s.Add_UInt32(__EDG_VERSION__ % 10);
+  #endif
+
   #ifdef __VERSION__
+    s.Add_Space();
+    s += "ver:";
     s += __VERSION__;
   #endif
 
   #ifdef __GNUC__
     s += " GCC ";
     s.Add_UInt32(__GNUC__);
-    s += '.';
+    s.Add_Dot();
     s.Add_UInt32(__GNUC_MINOR__);
-    s += '.';
+    s.Add_Dot();
     s.Add_UInt32(__GNUC_PATCHLEVEL__);
   #endif
 
-  #ifdef __clang__
-    s += " CLANG ";
-    s.Add_UInt32(__clang_major__);
-    s += '.';
-    s.Add_UInt32(__clang_minor__);
-  #endif
-
-  #ifdef __xlC__
-    s += " XLC ";
-    s.Add_UInt32(__xlC__ >> 8);
-    s += '.';
-    s.Add_UInt32(__xlC__ & 0xFF);
-    #ifdef __xlC_ver__
-      s += '.';
-      s.Add_UInt32(__xlC_ver__ >> 8);
-      s += '.';
-      s.Add_UInt32(__xlC_ver__ & 0xFF);
-    #endif
-  #endif
 
   #ifdef _MSC_VER
     s += " MSC ";
     s.Add_UInt32(_MSC_VER);
+    #ifdef _MSC_FULL_VER
+      s.Add_Dot();
+      s.Add_UInt32(_MSC_FULL_VER);
+    #endif
+      
   #endif
+
+    #if defined(__AVX512F__)
+      #if defined(__AVX512VL__)
+        #define MY_CPU_COMPILE_ISA "AVX512VL"
+      #else
+        #define MY_CPU_COMPILE_ISA "AVX512F"
+      #endif
+    #elif defined(__AVX2__)
+      #define MY_CPU_COMPILE_ISA "AVX2"
+    #elif defined(__AVX__)
+      #define MY_CPU_COMPILE_ISA "AVX"
+    #elif defined(__SSE2__)
+      #define MY_CPU_COMPILE_ISA "SSE2"
+    #elif defined(_M_IX86_FP) && (_M_IX86_FP >= 2)
+      #define MY_CPU_COMPILE_ISA "SSE2"
+    #elif defined(__SSE__)
+      #define MY_CPU_COMPILE_ISA "SSE"
+    #elif defined(_M_IX86_FP) && (_M_IX86_FP >= 1)
+      #define MY_CPU_COMPILE_ISA "SSE"
+    #elif defined(__i686__)
+      #define MY_CPU_COMPILE_ISA "i686"
+    #elif defined(__i586__)
+      #define MY_CPU_COMPILE_ISA "i586"
+    #elif defined(__i486__)
+      #define MY_CPU_COMPILE_ISA "i486"
+    #elif defined(__i386__)
+      #define MY_CPU_COMPILE_ISA "i386"
+    #elif defined(_M_IX86_FP)
+      #define MY_CPU_COMPILE_ISA "IA32"
+    #endif
+
+  AString s2;
+
+  #ifdef MY_CPU_COMPILE_ISA
+    s2.Add_OptSpaced(MY_CPU_COMPILE_ISA);
+  #endif
+
+#ifndef MY_CPU_ARM64
+  #ifdef __ARM_FP
+    s2.Add_OptSpaced("FP");
+  #endif
+  #ifdef __ARM_NEON
+    s2.Add_OptSpaced("NEON");
+  #endif
+  #ifdef __NEON__
+    s2.Add_OptSpaced("__NEON__");
+  #endif
+  #ifdef __ARM_FEATURE_SIMD32
+    s2.Add_OptSpaced("SIMD32");
+  #endif
+#endif
+
+  #ifdef __ARM_FEATURE_CRYPTO
+    s2.Add_OptSpaced("CRYPTO");
+  #endif
+
+  #ifdef __ARM_FEATURE_SHA2
+    s2.Add_OptSpaced("SHA2");
+  #endif
+
+  #ifdef __ARM_FEATURE_AES
+    s2.Add_OptSpaced("AES");
+  #endif
+
+  #ifdef __ARM_FEATURE_CRC32
+    s2.Add_OptSpaced("CRC32");
+  #endif
+
+  #ifdef __ARM_FEATURE_UNALIGNED
+    s2.Add_OptSpaced("UNALIGNED");
+  #endif
+
+
+  #ifdef MY_CPU_BE
+    s2.Add_OptSpaced("BE");
+  #endif
+
+  #if defined(MY_CPU_LE_UNALIGN) \
+      && !defined(MY_CPU_X86_OR_AMD64) \
+      && !defined(MY_CPU_ARM64)
+    s2.Add_OptSpaced("LE-unaligned");
+  #endif
+
+  if (!s2.IsEmpty())
+  {
+    s.Add_OptSpaced(": ");
+    s += s2;
+  }
 }
